.watermarked {
    position: relative;
}

.watermarked:after {
    content: "";
    display: block;
    width: 100%;
    height: 100%;
    position: absolute;
    top: 0px;
    left: 0px;
    background-image: url("https://image.shutterstock.com/image-vector/sold-out-red-grunge-stamp-260nw-626289539.jpg");
    background-size: 100px 100px;
    background-position: 30px 30px;
    background-repeat: no-repeat;
    opacity: 0.7;
}

.used > .watermarked:after {
    background-image: url("https://cdn1.vectorstock.com/i/1000x1000/13/40/used-grunge-rubber-stamp-vector-1811340.jpg");
}
